package com.cirmuller.maidaddition.threads;

package .tmp;

import com.cirmuller.maidaddition.Utils.CraftingTasks.CraftingTask;
import com.cirmuller.maidaddition.Utils.CraftingTasks.ItemList;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Stack;
import net.minecraft.core.BlockPos;
import net.minecraft.server.level.ServerLevel;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.crafting.Ingredient;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraftforge.common.capabilities.ForgeCapabilities;
import net.minecraftforge.common.util.LazyOptional;
import net.minecraftforge.items.IItemHandler;

public class CalculateCraftingStackThread extends Thread {
  List<BlockPos> chests;
  
  ItemList materialsHave;
  
  ServerLevel level;
  
  Stack<CraftingTask> tasks;
  
  ItemList materialsToGet;
  
  boolean completedFinished;
  
  public CalculateCraftingStackThread(ServerLevel level, List<BlockPos> chests, ItemList materialsToGet) {
    this.chests = chests;
    this.level = level;
    this.materialsToGet = materialsToGet;
    this.materialsHave = null;
    this.tasks = null;
    this.completedFinished = false;
  }
  
  public CalculateCraftingStackThread(ServerLevel level) {
    this(level, null, null);
    this.chests = new ArrayList<>();
    this.materialsToGet = new ItemList();
  }
  
  public CalculateCraftingStackThread(ServerLevel level, List<BlockPos> chests) {
    this(level, chests, null);
    this.materialsToGet = new ItemList();
  }
  
  public boolean addChest(BlockPos pos) {
    BlockEntity chest = this.level.m_7702_(pos);
    if (chest == null || !chest.getCapability(ForgeCapabilities.ITEM_HANDLER).isPresent())
      return false; 
    if (this.chests.contains(pos))
      return false; 
    this.chests.add(pos);
    return true;
  }
  
  public boolean addMaterialsToGet(ItemStack stack) {
    return this.materialsToGet.add(stack);
  }
  
  public boolean addMaterialsToGet(ItemList list) {
    return this.materialsToGet.addAll((Collection)list);
  }
  
  public Stack<CraftingTask> getCraftingTaskStack() {
    if (this.tasks != null)
      return this.tasks; 
    if (this.materialsHave == null)
      this.materialsHave = getItemList(this.chests); 
    this.tasks = new Stack<>();
    ItemList itemsLeft = this.materialsHave.copy();
    for (ItemStack itemStack : this.materialsToGet) {
      CraftingTask task = new CraftingTask(this.level, null, null, Ingredient.m_43927_(new ItemStack[] { itemStack }, ), Integer.valueOf(itemStack.m_41613_()));
      Stack<CraftingTask> tasks = task.createCraftingTasksStack(itemsLeft);
      if (tasks != null) {
        ((CraftingTask)tasks.get(0)).isTargetItem = true;
        this.tasks.addAll(tasks);
        for (CraftingTask craftingTask : tasks) {
          if (craftingTask.getItemToTakeOut() != null)
            itemsLeft.remove(craftingTask.getItemToTakeOut()); 
        } 
      } 
    } 
    this.completedFinished = true;
    return this.tasks;
  }
  
  public CraftingTask getLastTask() {
    return this.tasks.peek();
  }
  
  public void run() {
    getCraftingTaskStack();
  }
  
  public boolean isCompleted() {
    return this.completedFinished;
  }
  
  protected ItemList getItemList(List<BlockPos> materialChests) {
    ItemList result = new ItemList();
    for (Iterator<BlockPos> iterator = materialChests.iterator(); iterator.hasNext(); ) {
      BlockPos pos = iterator.next();
      BlockEntity chest = this.level.m_7702_(pos);
      if (chest == null)
        continue; 
      LazyOptional<IItemHandler> capability = chest.getCapability(ForgeCapabilities.ITEM_HANDLER);
      capability.ifPresent(cap -> {
            int sz = cap.getSlots();
            for (int i = 0; i < sz; i++)
              result.add(cap.getStackInSlot(i)); 
          });
    } 
    return result;
  }
  
  public List<BlockPos> getChests() {
    return this.chests;
  }
}
